@@ -50,7 +50,7 @@ use warnings;
 use strict;
 use 5.010;
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 use vars qw/$bopts/;
 
@@ -85,6 +85,8 @@ our @EXPORT = qw/
     drop_icmp
     open_service
     enable_ip_forwarding
+    drop_netbios
+    enable_state_matching
 /;
 
 =head2 initfw
@@ -103,6 +105,9 @@ sub initfw {
     # now the catchall filter, known as dropwall
     chain('new', { name => 'dropwall', jump => 'drop'});
     log_setup('dropwall', { prefix => 'Bison DropWall'});
+
+    # silent logging chain for all those annoying things
+    chain('new', { name => 'silent', jump => 'drop'});
 }
 
 sub has_ip_address {
@@ -149,6 +154,9 @@ Open ports to a service by name (www, ssh, ftp). If no arguments are passed
 it will open access to everyone. If you pass a hash with to => then the port 
 will be only available to that ip address.
 
+    open_service('ssh', { to => '10.1.1.5' }); # open 22 to 10.1.1.5 only
+    open_service('www'); # open port 80 to all
+
 =cut
 
 sub open_service {
@@ -256,6 +264,28 @@ sub chain {
     return 1;
 }
 
+=head2 drop_netbios
+
+We don't necessarily want netbios packets, so here's the 
+option to disable them.
+You can choose to log them silently, or loudly to the main 
+firewall chain.
+
+    drop_netbios();  # drop netbios silently
+    drop_netbios(1); # drop packets loudly by logging to firewall
+
+=cut
+
+sub drop_netbios {
+    my $loud = shift;
+
+    my $chain = 'silent';
+    if ($loud) { $chain = $bopts->{firewall}; }
+    
+    ipt("-A INPUT -p udp --sport 137 --dport 137 -j $chain");
+    return 1;
+}
+
 =head2 log_setup
 
 Sets up logging for a chain. You can specify the time, duration and prefix.
@@ -363,12 +393,17 @@ sub preroute {
         return 1;
     }
 }
-        
 
-sub iface {
-    my $iface = shift;
+=head2 enable_state_matching
 
-    say "-> Using interface $iface";
+Accept related and established connections so client side 
+activities, ie: ftp, work correctly.
+
+=cut
+
+sub enable_state_matching {
+    ipt("-A INPUT -m state --state RELATED,ESTABLISHED -j ACCEPT");
+    return 1;
 }
 
 =head2 enable_ip_forwarding
@@ -544,6 +579,8 @@ commands you need to generate your firewall script
 =cut
 
 sub bison_finish {
+    my $file = shift;
+
     my $errors = 0;
     if (@{$bopts->{errors}} > 0) { $errors = @{$bopts->{errors}} }
 
@@ -566,6 +603,18 @@ sub bison_finish {
             say "$i: $_";
         }
     }
+
+    # do we have a file to write to?
+    if ($file) {
+        open(my $fh, ">$file") or die "Could not write to $file\n";
+        print $fh "#########################\n";
+        print $fh "# Generated by Bison $VERSION\n";
+        print $fh "#########################\n";
+        for (@{$bopts->{buffer}}) {
+            print $fh $_ . "\n";
+        }
+        close $fh;
+    }
 }
 
 =head1 BUGS
